@@ -13,16 +13,25 @@ import uuid
 # https://stackoverflow.com/questions/15647580/does-null-have-a-data-type
 # https://stackoverflow.com/questions/7519621/where-is-null-not-working-in-sqlite
 # https://stackoverflow.com/questions/16856647/sqlite3-programmingerror-incorrect-number-of-bindings-supplied-the-current-sta
+# https://stackoverflow.com/questions/1535327/how-to-print-a-class-or-objects-of-class-using-print
 
 
 RESERVED_KEYS = "filename", "time"
 
 
+class Row(sqlite3.Row):
+
+    def __str__(self):
+        return '\n'.join(['{}: {}'.format(key, self[key]) for key in self.keys()])
+
+
 def _key_val_list(d, separate_nulls=False):
     keys = []
     vals = []
     null_list = []
     for key, val in d.items():
+        if not isinstance(val, (type(None), int, float, str, bytes)):
+            raise RuntimeError('{} has type {}, which is not supported'.format(key, type(val)))
         if separate_nulls and val is None:
             null_list.append(key)
         else:
@@ -36,7 +45,7 @@ def _key_val_list(d, separate_nulls=False):
 
 def _get_conn(db, wd, timeout=10):
     conn = sqlite3.connect(os.path.join(wd, db), timeout=timeout)
-    conn.row_factory = sqlite3.Row
+    conn.row_factory = Row
     with conn:
         conn.execute("create table if not exists filelist (filename text primary key not null, time timestamp)")
     return conn
@@ -78,7 +87,9 @@ def _make_expression_vals(metadata):
     return expr, vals
 
 
-def search(metadata, db="files.db", wd='.', timeout=10):
+def search(metadata, db="files.db", wd='.', timeout=10, verbose=False, keys_to_print=None):
+    if not os.path.exists(os.path.join(wd, db)):
+        raise FileNotFoundError('{} does not exist in {}'.format(db, wd))
     conn = _get_conn(db, wd, timeout=timeout)
     with conn:
         if len(metadata) > 0:
@@ -87,21 +98,14 @@ def search(metadata, db="files.db", wd='.', timeout=10):
             rows = conn.execute(query_string, vals).fetchall()
         else:
             rows = conn.execute("select * from filelist").fetchall()
+    if verbose:
+        _print_rows(rows, keys=keys_to_print)
     return rows
 
 
-def print_csv(db='files.db', wd='.', timeout=10):
-    rows = search({}, db=db, wd=wd, timeout=timeout)
-    first = True
-    for row in rows:
-        if first:
-            keys = list(row.keys())
-            print(','.join(keys))
-            first = False
-        print(','.join([str(row[key]) for key in keys]))
-
-
 def delete(metadata, db='files.db', wd='.', timeout=10, dryrun=False, delimiter='\t'):
+    if not os.path.exists(os.path.join(wd, db)):
+        raise FileNotFoundError('{} does not exist in {}'.format(db, wd))
     if len(metadata) == 0:
         raise ValueError("must have at least one search parameter")
     conn = _get_conn(db, wd, timeout=timeout)
@@ -130,11 +134,12 @@ def _parse_metadata(metadatalist):
     return metadata
 
 
-def _print_rows(rows, delimiter='\t'):
+def _print_rows(rows, delimiter='\t', keys=None):
     first = True
     for row in rows:
         if first:
-            keys = list(row.keys())
+            if keys is None:
+                keys = list(row.keys())
             print(delimiter.join(keys))
             first = False
         print(delimiter.join([str(row[key]) for key in keys]))
@@ -149,6 +154,7 @@ def main():
 
     parser_search = subparsers.add_parser('search', help='Search database')
     parser_search.add_argument('-d', '--delimiter', type=str, default='\t', help='Output column delimiter')
+    parser_search.add_argument('-o', '--output_columns', type=str, default=None, help='Comma delimited list of column names to print')
     parser_search.add_argument('metadata', nargs='*', help='list of keys and values', metavar="KEY=VALUE")
     parser_search.set_defaults(subcommand='search')
 
@@ -162,6 +168,7 @@ def main():
     parser_delete = subparsers.add_parser('delete', help='Delete files from database and working director')
     parser_delete.add_argument('-n', '--dry_run', action='store_true', help='Print entries to be delete, but do not delete')
     parser_delete.add_argument('-d', '--delimiter', type=str, default='\t', help='Output column delimiter for dry run')
+    parser_delete.add_argument('-o', '--output_columns', type=str, default=None, help='Comma delimited list of column names to print')
     parser_delete.add_argument('metadata', nargs='*', help='List of keys and values', metavar="KEY=VALUE")
     parser_delete.set_defaults(subcommand='delete')
 
@@ -174,7 +181,8 @@ def main():
         parser.print_help()
 
     elif args.subcommand == 'search':
-        _print_rows(search(_parse_metadata(args.metadata), db=args.db, wd=args.wd, timeout=args.timeout), delimiter=args.delimiter)
+        _print_rows(search(_parse_metadata(args.metadata), db=args.db, wd=args.wd, timeout=args.timeout), delimiter=args.delimiter,
+                    keys=None if args.output_columns is None else args.output_columns.split(','))
 
     elif args.subcommand == 'add':
         filename = add(_parse_metadata(args.metadata), db=args.db, wd=args.wd, filename=args.filename, timeout=args.timeout, ext=args.ext)
@@ -182,7 +190,8 @@ def main():
 
     elif args.subcommand == 'delete':
         rows = delete(_parse_metadata(args.metadata), db=args.db, wd=args.wd, timeout=args.timeout, dryrun=args.dry_run)
-        _print_rows(rows, delimiter=args.delimiter)
+        _print_rows(rows, delimiter=args.delimiter,
+                    keys=None if args.output_columns is None else args.output_columns.split(','))
 
     elif args.subcommand == 'test':
         import pytest
